use warnings;
use strict;

package Net::Domain::SMD;
use Log::Report   'net-domain-smd';

use MIME::Base64       qw/decode_base64/;
use XML::LibXML        ();
use POSIX              qw/mktime tzset/;
use XML::Compile::Util qw/type_of_node/;
use List::Util         qw/first/;
use Scalar::Util       qw/blessed/;
use DateTime           ();

=chapter NAME
Net::Domain::SMD - info from one SMD structure

=chapter SYNOPSIS

  # end-user route to collect the info
  my $tmch  = Net::Domain::TMCH->new(%options);
  my $smdf  = $tmch->smd($filename);
  my $smdf  = $tmch->smd($node);   # XML::LibXML::Element
  my $smdf  = $tmch->smd($string); # bytes

  # Data::Dumper is your friend
  foreach my $court ($smd->courts)
  {   print $court->{courtName}, "\n";
      print Dumper $court;
  }

  foreach my $trademark ($smd->trademarks)
  {   print $trademark->{markName}, "\n";
      my $holders = $trademark->{holder} || [];
      foreach my $holder (@$holders)
      {  print "Holder: $holder->{name}\n";
      }
  }

  foreach my $treaty ($smd->treaties) ...

=chapter DESCRIPTION
Draft RFC L<http://tools.ietf.org/html/draft-lozano-tmch-func-spec-06>
SMD information is packaged.  This object represents one file in wrapped
format.

=chapter METHODS

=section Constructors

=c_method new OPTIONS

=requires data HASH
=requires payload XML-NODE

=cut

sub new($%) { my ($class, %args) = @_; (bless {}, $class)->init(\%args) }
sub init($)
{   my ($self, $args) = @_;
    $self->{NDS_data}    = $args->{data}    or panic;
    $self->{NDS_payload} = $args->{payload} or panic;
    $self;
}

=c_method fromNode NODE, OPTIONS
Input is an M<XML::LibXML::Element>, which contains a signedMark structure.
The OPTIONS are passed on to M<new()>.

=requires schemas M<XML::Compile::Schema>
=cut

sub fromNode($%)
{   my ($class, $node, %args) = @_;
    my $schemas = delete $args{schemas} or panic;

    $node = $node->documentElement
        if $node->isa('XML::LibXML::Document');

    my $type = type_of_node $node;
    my $data = $schemas->reader($type)->($node);

    $class->new(payload => $node, data => $data, %args);
}

#----------------
=section Attributes

=method payload
Returns the M<XML::LibXML::Element> which contains the whole SMD
structure.

=cut

sub payload()   {shift->{NDS_payload}}
sub data()      {shift->{NDS_data}}      # avoid, undocumented
sub _mark()     {shift->data->{mark}}    # hidden

#----------------
=section XML content access

The following methods try to be smart, to hide complexities in the XML
structure, simplifying the main program.

=method courts
Returns a list of HASHes, each representing one court.
See F<examples/mark-templ.pl>

=example
  foreach my $court ($smd->courts)
  {   print $court->{courtName}, "\n";
      print Dumper $court;
  }
=cut

sub courts()  { @{shift->_mark->{court} || []} }

=method trademarks
Returns a list of HASHes, each representing one trademark.
See F<examples/mark-templ.pl>

=example
  foreach my $trademark ($smd->trademarks)
  {   print $trademark->{markName}, "\n";
      my $holders = $trademark->{holder} || [];
      foreach my $holder (@$holders)
      {  print "Holder: $holder->{name}\n";
      }
  }
=cut

sub trademarks()  { @{shift->_mark->{trademark} || []} }

=method treaties
Returns a list of HASHes, each representing one treaty or statute (one
element of type "treatyOrStatute".  See F<examples/mark-templ.pl>
=cut

sub treaties()  { @{shift->_mark->{treatyOrStatute} || []} }

=method certificates OPTIONS
Returns the certificates as contained in the SMD file.

=option  issuer NAME
=default issuer <any issuer>
Limit to the certificates of the issuer.
=cut

sub certificates(%)
{   my ($self, %args) = @_;

    my $tokens = $self->data->{ds_Signature}{ds_KeyInfo}{__TOKENS} || [];
    my @certs  = map $_->certificate, @$tokens;

    my $issuer = $args{issuer};
    $issuer ? (grep $_->issuer eq $issuer, @certs) : @certs;
}

=method issuer
Returns the issuer information, as a HASH.
=cut

sub issuer()
{   my $i = shift->data->{smd_issuerInfo} or return;
    # remove smd namespace prefixes
    my %issuer;
    while(my($k, $v) = each %$i)
    {   $k =~ s/smd_//;
        $issuer{$k} = $v;
    }
    \%issuer;
}

=method from
Returns the timestamp reflecting the 'notBefore' field, as string.
When "auto_datetime" is enabled, this will return a M<DateTime>
object.

=method fromTime
The value of M<from()> converted into seconds from epoch.

=method until
Returns the timestamp reflecting the 'notAfter' field, as string.
When "auto_datetime" is enabled, this will return a M<DateTime>
object.

=method untilTime
The value of M<until()> converted into seconds from epoc.

=cut

sub from()      {shift->data->{smd_notBefore}}
sub until()     {shift->data->{smd_notAfter}}
sub fromTime()  {my $s = shift; $s->date2time($s->from)->hires_epoch}
sub untilTime() {my $s = shift; $s->date2time($s->until)->hires_epoch}

=method smdID
=cut

sub smdID()     {shift->data->{smd_id}}


#----------------
=section Helpers

=ci_method date2time STRING
Converts a time string into a M<DateTime> object.  Accepted time-STRINGs
are like time stamps in the SMD file headers, but also the ISO values
inside an object.

=cut

sub date2time($)
{   my ($thing, $date) = @_;

    return $date
        if blessed $date && $date->isa('DateTime');

    # For now, I only support Zulu time: 2013-07-12T12:53:48.408Z
    $date =~ m/^ ([0-9]{4})\-([0-1]?[0-9])\-([0-3]?[0-9])
               T ([0-2]?[0-9])\:([0-5]?[0-9])\:([0-6]?[0-9])(\.[0-9]+)?
               ([+-][0-9]?[0-9]\:[0-9][0-9]|Z)? $/x
        or return;

    DateTime->new
      ( year => $1, month => $2, day => $3
      , hour => $4, minute => $5, second => $6, 
      , nanosecond => int(1_000_000_000 * ($7 || 0))
      , time_zone  => ($8 || 'UTC')
      );
}

1;
