use warnings;
use strict;

package Net::Domain::TMCH::CRL;
use base 'Exporter';

use Log::Report    'net-domain-smd';
use MIME::Base64   qw(decode_base64);
use Convert::X509  ();
use Scalar::Util   qw(blessed);

=chapter NAME
Net::Domain::TMCH::CRL - TMCH Certificates Revokation List

=chapter SYNOPSIS

  # End-user interface

  my $tmch = Net::Domain::TMCH->new
    ( ...
    , cert_revocations => $fn    # or $uri
    );
  $crl = $tmch->crl;

  # create explicitly

  my $crl  = Net::Domain::TMCH::CRL->fromFile($fn, %options);
  my $crl  = Net::Domain::TMCH::CRL->fromURI($uri, %options);
  my $crl  = Net::Domain::TMCH::CRL->new(source => 'hardcoded'
    , revoked => \@serials);
  my $tmch = Net::Domain::TMCH->new(..., cert_revocations => $crl);

  if($crl->isRevoked($certificate)) ...
  if($crl->isRevoked($certificate->serial)) ...

  
=chapter DESCRIPTION
Draft RFC L<http://tools.ietf.org/html/draft-lozano-tmch-func-07>
specifies a way for TLD (domain registries) to exchange trademark rights
(brand registrations).  This module manages the revocation list for TMV
(Trademark Validator) certificates.

=chapter METHODS

=section Constructors

=c_method new OPTIONS

=requires source STRING|FILENAME|URI
Identification for the source of this list, to be used in error message
and such.

=option  revoked HASH|ARRAY
=default revoked []
ARRAY of serial numbers from revoked certificates, or a HASH of which the
keys are the revoked serials.
=cut


sub new($%) { my ($class, %args) = @_; (bless {}, $class)->init(\%args) }

sub init($)
{   my ($self, $args) = @_;
    $self->{NDTC_source}  = $args->{source} or panic;

    my $rev =  $args->{revoked} || [];
    $rev = +{ map +($_ => 1), @$rev} if ref $rev eq 'ARRAY';
    $self->{NDTC_revoked} = $rev;
    $self;
}

=method fromFile FILENAME, OPTIONS
Read a CRL from file, for instance a cached or test CRL.
The OPTIONS are passed to M<new()>.
=cut

sub fromFile($%)
{   my ($class, $fn) = (shift, shift);

    open my($fh), '<:raw', $fn
        or fault __x"cannot read CRL file {fn}", fn => $fn;

    my $crl = Convert::X509::CRL->new(join '', $fh->getlines);
    $class->new(source => $fn, revoked => $crl->{crl}, @_);
}

=method fromString STRING, OPTIONS
Read a CRL from STRING, for instance a cached or test CRL.
The OPTIONS are passed to M<new()>.
=cut

sub fromString($%)
{   my $class = shift;
    my $crl = Convert::X509::CRL->new(shift);
    $class->new(source => 'string', revoked => $crl->{crl}, @_);
}

=method fromURI URI, OPTIONS
Create a new CRL object from an URI, which is collected via LWP.  The
OPTIONS are passed to M<new()>.
=cut

my $ua;
sub fromURI($%)
{   my ($class, $uri) = (shift, shift);

    eval "require LWP::UserAgent";
    $@ and error __x"need LWP::UserAgent to fetch CRL: {err}", err => $@;

    $ua ||= LWP::UserAgent->new;
    my $resp = $ua->get($uri);
    $resp->is_success
        or error __x"could not collect CRL from {source}: {err}"
             , source => $uri, err => $resp->status_line;

    my $crl = Convert::X509::CRL->new($resp->decoded_content);
    $class->new(source => $uri, revoked => $crl->{crl}, @_);
}

#-------------------------

=section Attributes
=method source
=cut

sub source() {shift->{NDTC_source}}

#-------------------------

=section Actions

=cut

=method isRevoked CERTIFICATE|SERIAL
Checks whether a CERTIFICATE (M<Crypt::OpenSSL::X509> object) has been
revoked.  It is sufficient to provide a SERIAL number of the certificate.
=cut

sub isRevoked($)
{   my ($self, $cert) = @_;
    my $serial = blessed $cert ? $cert->serial : $cert;
    exists $self->{NDTC_revoked}{lc $serial};
}

1;
